import { BinaryCoStream, CoMap, co, subscriptionsScopes, } from "../../internal.js";
/** @category Media */
export class ImageDefinition extends CoMap {
    originalSize = co.json();
    placeholderDataURL = co.string;
    [co.items] = co.ref(BinaryCoStream);
    highestResAvailable(options) {
        if (!subscriptionsScopes.get(this)) {
            console.warn("highestResAvailable() only makes sense when used within a subscription.");
        }
        const resolutions = Object.keys(this).filter((key) => key.match(/^\d+x\d+$/) &&
            (options?.maxWidth === undefined ||
                Number(key.split("x")[0]) <= options.maxWidth));
        resolutions.sort((a, b) => {
            const aWidth = Number(a.split("x")[0]);
            const bWidth = Number(b.split("x")[0]);
            return aWidth - bWidth;
        });
        let highestAvailableResolution;
        for (const resolution of resolutions) {
            if (this[resolution] && this[resolution]?.getChunks()) {
                highestAvailableResolution = resolution;
            }
            else {
                return (highestAvailableResolution && {
                    res: highestAvailableResolution,
                    stream: this[highestAvailableResolution],
                });
            }
        }
        return (highestAvailableResolution && {
            res: highestAvailableResolution,
            stream: this[highestAvailableResolution],
        });
    }
}
//# sourceMappingURL=imageDef.js.map