import { Account, CoMap, CoValueBase, Ref, co, isControlledAccount, AccountAndGroupProxyHandler, MembersSym, loadCoValue, subscribeToCoValue, ensureCoValueLoaded, subscribeToExistingCoValue, } from "../internal.js";
/** @category Identity & Permissions */
export class Profile extends CoMap {
    name = co.string;
}
/** @category Identity & Permissions */
export class Group extends CoValueBase {
    static {
        this.prototype._type = "Group";
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    static _schema;
    get _schema() {
        return this.constructor._schema;
    }
    static {
        this._schema = {
            profile: "json",
            root: "json",
            [MembersSym]: {
                ref: () => Account,
                optional: false,
            },
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
        };
        Object.defineProperty(this.prototype, "_schema", {
            get: () => this._schema,
        });
    }
    get _refs() {
        const profileID = this._raw.get("profile");
        const rootID = this._raw.get("root");
        return {
            profile: profileID &&
                new Ref(profileID, this._loadedAs, this._schema.profile),
            root: rootID &&
                new Ref(rootID, this._loadedAs, this._schema.root),
        };
    }
    /** @deprecated Don't use constructor directly, use .create */
    constructor(options) {
        super();
        let raw;
        if (options && "fromRaw" in options) {
            raw = options.fromRaw;
        }
        else {
            const initOwner = options.owner;
            if (!initOwner)
                throw new Error("No owner provided");
            if (initOwner._type === "Account" &&
                isControlledAccount(initOwner)) {
                const rawOwner = initOwner._raw;
                raw = rawOwner.createGroup();
            }
            else {
                throw new Error("Can only construct group as a controlled account");
            }
        }
        Object.defineProperties(this, {
            id: {
                value: raw.id,
                enumerable: false,
            },
            _raw: { value: raw, enumerable: false },
        });
        return new Proxy(this, AccountAndGroupProxyHandler);
    }
    static create(options) {
        return new this(options);
    }
    myRole() {
        return this._raw.myRole();
    }
    addMember(member, role) {
        this._raw.addMember(member === "everyone" ? member : member._raw, role);
        return this;
    }
    get members() {
        return this._raw
            .keys()
            .filter((key) => {
            return key === "everyone" || key.startsWith("co_");
        })
            .map((id) => {
            const role = this._raw.get(id);
            const accountID = id === "everyone"
                ? undefined
                : id;
            const ref = accountID &&
                new Ref(accountID, this._loadedAs, this._schema[MembersSym]);
            const accessRef = () => ref?.accessFrom(this, "members." + id);
            return {
                id: id,
                role,
                ref,
                get account() {
                    return accessRef();
                },
            };
        });
    }
    /** @category Subscription & Loading */
    static load(id, as, depth) {
        return loadCoValue(this, id, as, depth);
    }
    /** @category Subscription & Loading */
    static subscribe(id, as, depth, listener) {
        return subscribeToCoValue(this, id, as, depth, listener);
    }
    /** @category Subscription & Loading */
    ensureLoaded(depth) {
        return ensureCoValueLoaded(this, depth);
    }
    /** @category Subscription & Loading */
    subscribe(depth, listener) {
        return subscribeToExistingCoValue(this, depth, listener);
    }
}
//# sourceMappingURL=group.js.map