export const subscriptionsScopes = new WeakMap();
const TRACE_INVALIDATIONS = false;
export class SubscriptionScope {
    scopeID = `scope-${Math.random().toString(36).slice(2)}`;
    subscriber;
    entries = new Map();
    rootEntry;
    onUpdate;
    scheduledUpdate = false;
    cachedValues = {};
    parents = {};
    constructor(root, rootSchema, onUpdate) {
        this.rootEntry = {
            state: "loaded",
            value: root,
            rawUnsub: () => { }, // placeholder
        };
        this.entries.set(root.id, this.rootEntry);
        subscriptionsScopes.set(root, this);
        this.subscriber = root._loadedAs;
        this.onUpdate = onUpdate;
        this.rootEntry.rawUnsub = root._raw.core.subscribe((rawUpdate) => {
            if (!rawUpdate)
                return;
            this.rootEntry.value = rootSchema.fromRaw(rawUpdate);
            // console.log("root update", this.rootEntry.value.toJSON());
            subscriptionsScopes.set(this.rootEntry.value, this);
            this.scheduleUpdate();
        });
    }
    scheduleUpdate() {
        if (!this.scheduledUpdate) {
            this.scheduledUpdate = true;
            queueMicrotask(() => {
                this.scheduledUpdate = false;
                this.onUpdate(this.rootEntry.value);
            });
        }
    }
    onRefAccessedOrSet(fromId, accessedOrSetId) {
        // console.log("onRefAccessedOrSet", this.scopeID, accessedOrSetId);
        if (!accessedOrSetId) {
            return;
        }
        this.parents[accessedOrSetId] =
            this.parents[accessedOrSetId] || new Set();
        this.parents[accessedOrSetId].add(fromId);
        if (!this.entries.has(accessedOrSetId)) {
            const loadingEntry = {
                state: "loading",
                immediatelyUnsub: false,
            };
            this.entries.set(accessedOrSetId, loadingEntry);
            const node = this.subscriber._type === "Account"
                ? this.subscriber._raw.core.node
                : this.subscriber.node;
            void node.loadCoValueCore(accessedOrSetId).then((core) => {
                if (loadingEntry.state === "loading" &&
                    loadingEntry.immediatelyUnsub) {
                    return;
                }
                if (core !== "unavailable") {
                    const entry = {
                        state: "loaded",
                        rawUnsub: () => { }, // placeholder
                    };
                    this.entries.set(accessedOrSetId, entry);
                    const rawUnsub = core.subscribe((rawUpdate) => {
                        // console.log("ref update", this.scopeID, accessedOrSetId, JSON.stringify(rawUpdate))
                        if (!rawUpdate)
                            return;
                        this.invalidate(accessedOrSetId);
                        this.scheduleUpdate();
                    });
                    entry.rawUnsub = rawUnsub;
                }
            });
        }
    }
    invalidate(id, fromChild, seen = new Set()) {
        if (seen.has(id))
            return;
        TRACE_INVALIDATIONS &&
            console.log("invalidating", fromChild, "->", id, this.cachedValues[id]);
        delete this.cachedValues[id];
        seen.add(id);
        for (const parent of this.parents[id] || []) {
            this.invalidate(parent, id, seen);
        }
    }
    unsubscribeAll() {
        for (const entry of this.entries.values()) {
            if (entry.state === "loaded") {
                entry.rawUnsub();
            }
            else {
                entry.immediatelyUnsub = true;
            }
        }
        this.entries.clear();
    }
}
//# sourceMappingURL=subscriptionScope.js.map